function noop() {
}

function getPlayer() {
  const { player } = this.$refs.video;
  return player;
}

function setTimestamp(timestamp) {
  const player = getPlayer.call(this);
  const duration = this.end - this.start;
  const isPlaying = !player.paused();
  player.pause();
  player.currentTime(Math.min(Math.max(timestamp, 0), duration));
  if (isPlaying) {
    player.play().catch(noop);
  }
}

function onNumberKey(key) {
  return function toggleActivity() {
    this.$refs.form.toggleActivity(key);
  };
}

export default {
  '?': function openHelp() {
    this.helpDialog = true;
  },
  /* video shortcuts */
  ' ': function playPause() {
    const player = getPlayer.call(this);
    if (player.paused()) {
      player.play().catch(noop);
    } else {
      player.pause();
    }
  },
  '[': function slowDown() {
    const player = getPlayer.call(this);
    player.playbackRate(player.playbackRate() / 2);
  },
  ']': function speedUp() {
    const player = getPlayer.call(this);
    player.playbackRate(player.playbackRate() * 2);
  },
  '\\': function defaultSpeed() {
    const player = getPlayer.call(this);
    player.playbackRate(1);
  },
  ArrowLeft() {
    const player = getPlayer.call(this);
    setTimestamp.call(this, player.currentTime() - 0.5);
  },
  ArrowRight() {
    const player = getPlayer.call(this);
    setTimestamp.call(this, player.currentTime() + 0.5);
  },
  ArrowDown() {
    const player = getPlayer.call(this);
    setTimestamp.call(this, player.currentTime() - 5);
  },
  ArrowUp() {
    const player = getPlayer.call(this);
    setTimestamp.call(this, player.currentTime() + 5);
  },
  /* form shortcuts */
  1: onNumberKey(1),
  2: onNumberKey(2),
  3: onNumberKey(3),
  4: onNumberKey(4),
  5: onNumberKey(5),
  6: onNumberKey(6),
  7: onNumberKey(7),
  8: onNumberKey(8),
  9: onNumberKey(9),
  Enter(evt) {
    if (evt.shiftKey) {
      this.$refs.form.submit();
    } else {
      return null;
    }
    return true;
  },
};

