#!/usr/bin/env python
import click
import json
import os
from stumpf.diva_models import Detection, Activity, Track, ActivityTrack

from stumpfwald import cli, authorized
from stumpfwald.task import generate

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

stumpf_postgres_uri = os.getenv('STUMPF_POSTGRES_URI', default='postgresql://stumpf@localhost/stumpf')
engine = create_engine(stumpf_postgres_uri)
Session = sessionmaker(bind=engine)
session = Session()

batch = 'batch1'
q_batch_size = 10
# app_id = '4cfe76d6-e7ee-4f3b-a17d-8473835f73dd'  # dev
app_id = '0cd78586-06e9-4ae4-9d46-2e5a98855fe3'
videos = [
    '8292fe8c-3b73-49cf-820b-686c34f5e634',  # 2018-03-15_10-35-01_10-40-01_G508_bus
]
title = 'Decide if two people are the same'
description = 'Look at two still shots and determine whether both feature the same person.'
sandbox = False
max_assignments = 3
duration =  10 * 60
lifetime = 12 * 60 * 60
reward = 10

def closest_detection(detections):
    closest = None
    axis = 3
    for d in detections:
        if (closest is None or d.bounds[axis] > closest.bounds[axis]):
            closest = d
    return closest


def project_fields(d):
    return {
        'bounds': d.bounds,
        'id': str(d.id),
        'frame': d.frame
    }


def all_pairs(list):
    pairs = []
    for i, value in enumerate(list):
        for j in range(i + 1, len(list)):
            pairs.append((value, list[j]))
    return pairs


# This function will eventually be much more complex.
def generate_args(video):
    entities = []
    for t in session.query(Track).\
            filter(Track.video_id == video).\
            filter(Track.created > '10-03-2018').\
            all():
        detections = session.query(Detection).filter(Detection.track_id == t.id).all()
        entities.append(project_fields(closest_detection(detections)))
    pairs = all_pairs(entities)
    groups = []
    for k in range(0, len(pairs), 10):
        upper = len(pairs) if k+10 > len(pairs) else k+10
        group = pairs[k:upper]
        groups.append(group)
    return groups


@cli.command(name='generate-kwits')
@click.pass_context
def genKwits(ctx):
    with open(batch + '.kwits.jsonl', 'w') as output:
        for v in videos:
            for pairset in generate_args(v):
                ctx.invoke(generate,
                           type='KWIT',
                           video_id=v,
                           app_id=app_id,
                           args=json.dumps({
                                'sandbox': True,
                                'title': title,
                                'description': description,
                                'args': {
                                    'pairs': pairset
                                },
                                'max_assignments': 1,
                           }),
                           output=output)


@cli.command(name='generate-hits')
@click.pass_context
def genHits(ctx):
    with open(batch + '.hits.jsonl', 'w') as output:
        for v in videos:
            for pairset in generate_args(v):
                ctx.invoke(generate,
                           type='HIT',
                           video_id=v,
                           app_id=app_id,
                           args=json.dumps({
                                'sandbox': sandbox,
                                'args': {
                                    'pairs': pairset
                                },
                                'title': title,
                                'description': description,
                                'max_assignments': max_assignments,
                                'assignment_duration_in_seconds': duration,
                                'lifetime_in_seconds': lifetime,
                                'reward': reward,
                                'auto_approval_delay_in_seconds': 10
                           }),
                           output=output)


if __name__ == '__main__':
    cli.main()
