"""
Drop video.channel field

Revision ID: 0a460585abe4
Revises: 5078f97311a1
Create Date: 2018-08-17 12:37:53.670773

"""
import csv
import os
import tempfile

from alembic import op
import boto3
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '0a460585abe4'
down_revision = '5078f97311a1'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('video', 'channel')
    # ### end Alembic commands ###
    fiftyseven_videos_path = 'next_fifty.csv'
    with tempfile.TemporaryDirectory() as tmpdirname:
        # Download csv from s3. Requires boto credentials to be set.
        BUCKET_NAME = 'diva-mturk-video-metadata'
        local_fiftyseven_videos_path = os.path.join(tmpdirname, fiftyseven_videos_path)
        s3 = boto3.resource('s3')
        s3.Bucket(BUCKET_NAME).download_file(fiftyseven_videos_path, local_fiftyseven_videos_path)

        query_template = """insert into video
(created,
 id, path, name, frame_rate, duration, width, height, camera_gtag, node, start_time, end_time)
                   values
(CURRENT_TIMESTAMP, '%s', '%s',
'%s', %s, %s, %s, %s, '%s', '%s', """
        query_template += "to_timestamp('%s', 'YYYY-MM-DD HH24:MI:SS') at time zone 'UTC-4', "
        query_template += "to_timestamp('%s', 'YYYY-MM-DD HH24:MI:SS') at time zone 'UTC-4')"

        with open(local_fiftyseven_videos_path, newline='') as csvfile:
            reader = csv.DictReader(csvfile)
            for row in reader:
                values = [item[1] for item in list(row.items())]
                query = query_template % (tuple(values))
                op.execute(query)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('video', sa.Column('channel', sa.Integer(), nullable=True))
    # ### end Alembic commands ###
