"""
Create table for qualification types

Revision ID: 1aa0d3e15fac
Revises: b56de99bc3c5
Create Date: 2018-09-06 09:37:01.665911

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '1aa0d3e15fac'
down_revision = 'b56de99bc3c5'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('qualification_type',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('user_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('mturk_qt_id', sa.String(), nullable=True),
                    sa.Column('sandbox', sa.BOOLEAN(), nullable=False),
                    sa.Column('name', sa.String(), nullable=False),
                    sa.Column('description', sa.String(), nullable=True),
                    sa.Column('keywords', sa.String(), nullable=True),
                    sa.Column('status', postgresql.ENUM('Active', 'Inactive',
                                                        name='qualification_type_status_enum'),
                              nullable=False),
                    sa.ForeignKeyConstraint(['user_id'], ['user.id'],
                                            name=op.f('fk_qualification_type_user_id_user')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_qualification_type')),
                    sa.UniqueConstraint('mturk_qt_id',
                                        name=op.f('uq_qualification_type_mturk_qt_id')),
                    sa.UniqueConstraint('name', name=op.f('uq_qualification_type_name')))
    op.add_column('task', sa.Column('qualification_requirements',
                                    postgresql.JSONB(astext_type=sa.Text()), nullable=True))
    op.execute("UPDATE task SET qualification_requirements = '[]'::jsonb")
    op.alter_column('task', 'qualification_requirements', nullable=False)

    op.add_column('task', sa.Column('qualification_for_id',
                                    postgresql.UUID(as_uuid=True), nullable=True))
    op.create_foreign_key(op.f('fk_task_qualification_for_id_qualification_type'),
                          'task', 'qualification_type', ['qualification_for_id'], ['id'])


def downgrade():
    op.drop_constraint(op.f('fk_task_qualification_for_id_qualification_type'),
                       'task', type_='foreignkey')
    op.drop_column('task', 'qualification_for_id')
    op.drop_table('qualification_type')
    op.drop_column('task', 'qualification_requirements')
