"""
improve arity answer view

Revision ID: 3305f15a054e
Revises: 6a8cf80617d1
Create Date: 2018-09-01 14:52:37.364620

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = '3305f15a054e'
down_revision = '6a8cf80617d1'
branch_labels = None
depends_on = None


def upgrade():
    # We are no longer using the assignment answer view because it
    # complicates things too much.
    op.execute("""DROP VIEW IF EXISTS view_arity_answers CASCADE""")

    # Create a function that gets an arity activity name from an activity_type_enum
    # (This should be replaced by referencing the enum's directly in arity.)
    op.execute("""
        CREATE OR REPLACE FUNCTION arity_type_to_activity(activity_type_enum) RETURNS text AS
        $$
            SELECT name FROM arity_activity WHERE type_ = $1 LIMIT 1;
        $$ LANGUAGE SQL;
    """)

    # Create an enumerated type that describes explicitly the answers generated
    # by arity.
    op.execute("""
        DROP TYPE IF EXISTS arity_adjudication_enum CASCADE;
        CREATE TYPE arity_adjudication_enum as ENUM (
            'No aggreement', '0 times', '1 time', '2+ times'
        );
    """)

    # Create a function that generates an adjudicated answer enum from an activity
    # type.  This distinguishes between having no agreement and the activity not
    # being present in the task (NULL).
    op.execute("""
        CREATE OR REPLACE
        FUNCTION arity_answer_to_adjudication_enum(
            activity_type activity_type_enum, answer jsonb, args jsonb)
        RETURNS arity_adjudication_enum AS $$
            DECLARE
                ret arity_adjudication_enum;
                activity_answer text;
                activity text;
                arg_substring int;
            BEGIN
            activity = arity_type_to_activity(activity_type);
            activity_answer = jsonb_extract_path_text(answer, activity);
            arg_substring = position(
                '"' || activity || '":' IN
                jsonb_extract_path_text(args, 'activities')
            );

            IF(arg_substring = 0) THEN
                ret = Null;
            ELSIF(activity_answer = '0') THEN
                ret = '0 times';
            ELSIF(activity_answer = '1') THEN
                ret = '1 time';
            ELSIF(activity_answer = '2') THEN
                ret = '2+ times';
            END IF;
            RETURN ret;
            END
        $$ LANGUAGE plpgsql IMMUTABLE;
    """)

    # Create a view from the task table containing only adjudicated arity tasks with
    # each activity as a column.
    # WARNING: This statement will need to be rerun if the activity_type_enum ever
    # changes.
    op.execute("""
        DO
        $$
            DECLARE answer_keys text;
        BEGIN
            DROP VIEW IF EXISTS view_arity_task_answers CASCADE;
            SELECT string_agg(
                    format(
                        'arity_answer_to_adjudication_enum(%L, answer, args) as %I', type_, type_),
                ', ')
                INTO answer_keys
                FROM unnest(enum_range(null::activity_type_enum)) AS type_;
            EXECUTE 'CREATE VIEW view_arity_task_answers AS SELECT '
                || 'id,created,video_id,'
                || answer_keys
                || ' FROM task '
                || '   WHERE annotation_app_id = ''2d755948-af16-40bf-846e-0ce885a43b53'''
                || '   AND answer IS NOT NULL'; /* adjudicated tasks */
        END;
        $$
        ;

    """)


def downgrade():
    # Clean up the junk created in the upgrade.
    op.execute("""
        DROP FUNCTION IF EXISTS arity_type_to_activity;
        DROP TYPE IF EXISTS arity_adjudication_enum CASCADE;
        DROP FUNCTION IF EXISTS arity_answer_to_adjudication_enum;
        DROP VIEW IF EXISTS view_arity_task_answers CASCADE;
    """)
