"""
Update functions and triggers for tama

Revision ID: 39fc11265800
Revises: 9b77d1c68a4f
Create Date: 2018-09-26 17:27:21.430115

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = '39fc11265800'
down_revision = '9b77d1c68a4f'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
             CREATE OR REPLACE FUNCTION log_notify()
             RETURNS TRIGGER AS
             $BODY$
             BEGIN
                     IF NOT NEW.message ? 'channel' THEN
                        RAISE EXCEPTION 'NOTIFY messages must contain a "channel" attribute';
                     END IF;
                     IF NOT NEW.message ? 'payload' THEN
                        RAISE EXCEPTION 'NOTIFY messages must contain a "payload" attribute';
                     END IF;

                     PERFORM pg_notify('log:INSERT'::text, NEW.id::text);
                     RETURN NEW;
             END;
             $BODY$
             LANGUAGE plpgsql VOLATILE;""")
    op.execute('DROP TRIGGER IF EXISTS log_notify ON log;')
    op.execute("""
             CREATE TRIGGER log_notify
             AFTER INSERT ON log
             FOR EACH ROW WHEN (NEW.category = 'NOTIFY')
             EXECUTE PROCEDURE log_notify();""")


def downgrade():
    op.execute("""
            CREATE OR REPLACE FUNCTION log_notify()
            RETURNS TRIGGER AS
            $BODY$
            BEGIN
                    IF NOT NEW.message ? 'channel' THEN
                    RAISE EXCEPTION 'NOTIFY messages must contain a "channel" attribute';
                    END IF;
                    IF NOT NEW.message ? 'payload' THEN
                    RAISE EXCEPTION 'NOTIFY messages must contain a "payload" attribute';
                    END IF;

                    PERFORM pg_notify(NEW.message ->> 'channel', NEW.message ->> 'payload');
                    RETURN NEW;
            END;
            $BODY$
            LANGUAGE plpgsql VOLATILE;""")
    op.execute('DROP TRIGGER IF EXISTS log_notify ON log;')
    op.execute("""
            CREATE TRIGGER log_notify
            BEFORE INSERT ON log
            FOR EACH ROW WHEN (NEW.category = 'NOTIFY')
            EXECUTE PROCEDURE log_notify();""")
