"""
add fact table and provenance

Revision ID: 3d0fd8f71c20
Revises: a84f9e93ed15
Create Date: 2018-10-16 09:42:31.367371

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '3d0fd8f71c20'
down_revision = 'a84f9e93ed15'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('fact',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('video_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('frame_range', postgresql.INT4RANGE(), nullable=True),
                    sa.Column('fact', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
                    sa.ForeignKeyConstraint(['video_id'], ['video.id'],
                                            name=op.f('fk_fact_video_id_video')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_fact')))
    op.create_index(op.f('ix_fact_video_id'), 'fact', ['video_id'], unique=False)
    op.create_table('provenance',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('fact_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('task_id', postgresql.UUID(as_uuid=True), nullable=True),
                    sa.Column('relationship', postgresql.ENUM(
                        'Task_Created_Fact',
                        'External_Process_Created_Fact',
                        'Batch_Process_Created_Fact',
                        name='provenance_relationship'), nullable=False),
                    sa.Column('meta', postgresql.JSONB(astext_type=sa.Text()), nullable=False),
                    sa.CheckConstraint(
                        "relationship != 'Task_Created_Fact' OR task_id IS NOT NULL",
                        name=op.f('ck_provenance_task_id_not_null_if_task_created_Fact')),
                    sa.ForeignKeyConstraint(['fact_id'], ['fact.id'],
                                            name=op.f('fk_provenance_fact_id_fact')),
                    sa.ForeignKeyConstraint(['task_id'], ['task.id'],
                                            name=op.f('fk_provenance_task_id_task')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_provenance')))
    op.create_index(op.f('ix_provenance_fact_id'), 'provenance', ['fact_id'], unique=False)
    op.create_index(op.f('ix_provenance_task_id'), 'provenance', ['task_id'], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_provenance_task_id'), table_name='provenance')
    op.drop_index(op.f('ix_provenance_fact_id'), table_name='provenance')
    op.drop_table('provenance')
    op.drop_index(op.f('ix_fact_video_id'), table_name='fact')
    op.drop_table('fact')
    # ### end Alembic commands ###
