"""
Add views for metabase

Revision ID: 666af989350a
Revises: 60e51dc64d5f
Create Date: 2018-08-14 15:08:03.077357

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = '666af989350a'
down_revision = '60e51dc64d5f'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
             CREATE OR REPLACE VIEW view_assignment_status_times AS (
                SELECT
                assignment_id, accepted, submitted, returned, abandoned, approved,
                EXTRACT(epoch from (submitted - accepted)) as tt_submitted,
                EXTRACT(epoch from (approved - accepted))  as tt_approved,
                EXTRACT(epoch from (returned - accepted))  as tt_returned,
                EXTRACT(epoch from (abandoned - accepted))  as tt_abandoned
                FROM
                CROSSTAB('
                    SELECT cross_product.assignment_id, cross_product.status, s.occurred
                    FROM (
                    SELECT DISTINCT assignment_id, statuses.status
                    FROM assignment_status_change
                    CROSS JOIN (
                        SELECT DISTINCT status FROM assignment_status_change
                        ) AS statuses
                    ) AS cross_product
                    LEFT OUTER JOIN assignment_status_change AS s
                    ON cross_product.assignment_id = s.assignment_id AND
                       cross_product.status = s.status
                    ORDER BY 1, 2
                ') AS (assignment_id UUID,
                    abandoned     TIMESTAMP,
                    returned      TIMESTAMP,
                    accepted      TIMESTAMP,
                    submitted     TIMESTAMP,
                approved TIMESTAMP));

             CREATE OR REPLACE VIEW view_assignment_detail AS (
                SELECT vast.assignment_id,
                    vast.accepted,
                    vast.submitted,
                    vast.returned,
                    vast.abandoned,
                    vast.approved,
                    vast.tt_submitted,
                    vast.tt_approved,
                    vast.tt_returned,
                    vast.tt_abandoned,
                    a.created AS assignment_created,
                    a.status AS assignment_status,
                    a.worker_id,
                    a.app_version,
                    aa.id AS app_id,
                    aa.name AS app_name,
                    t.id AS task_id,
                    t.title AS task_title,
                    t.max_assignments,
                    t.status AS task_status,
                    t.reward,
                    t.sandbox,
                    t.assignment_duration_in_seconds,
                    v.id AS video_id,
                    v.path,
                    v.frame_rate,
                    v.duration
                FROM view_assignment_status_times vast,
                    assignment a,
                    annotation_app aa,
                    task t,
                    video v
                WHERE a.task_id = t.id AND
                        v.id = t.video_id AND
                        aa.id = t.annotation_app_id AND
                        vast.assignment_id = a.id);

             CREATE OR REPLACE VIEW view_assignment_status_change_detail AS (
                SELECT a.id,
                    a.created AS assignment_created,
                    asc_.status AS assignment_status,
                    a.worker_id,
                    a.app_version,
                    aa.id AS app_id,
                    aa.name AS app_name,
                    t.id AS task_id,
                    t.title AS task_title,
                    t.max_assignments,
                    t.status AS task_status,
                    t.reward,
                    t.sandbox,
                    t.assignment_duration_in_seconds,
                    v.id AS video_id,
                    v.path,
                    v.frame_rate,
                    v.duration
                FROM assignment_status_change asc_,
                    assignment a,
                    annotation_app aa,
                    task t,
                    video v
                WHERE a.task_id = t.id AND
                        v.id = t.video_id AND
                        aa.id = t.annotation_app_id AND
                        asc_.assignment_id = a.id);""")


def downgrade():
    op.execute("""
    DROP VIEW IF EXISTS view_assignment_status_time, view_assignment_detail,
    view_assignment_status_change_detail""")
