"""
Create Tama triggers

Revision ID: 6a8cf80617d1
Revises: 7321a6c8691f
Create Date: 2018-08-29 08:46:00.537499

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = '6a8cf80617d1'
down_revision = '7321a6c8691f'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
CREATE OR REPLACE FUNCTION log_insert_update_or_delete() RETURNS TRIGGER AS $BODY$
DECLARE
channel TEXT;
payload JSONB;
ret RECORD;
BEGIN

channel = TG_TABLE_NAME || ':' || TG_OP;

IF(TG_OP = 'DELETE') THEN
  payload = json_build_object('old', to_jsonb(OLD));
  ret = OLD;
ELSIF (TG_OP = 'UPDATE') THEN
  payload = json_build_object('old', to_jsonb(OLD), 'new', to_jsonb(NEW));
  ret = NEW;
ELSIF (TG_OP = 'INSERT') THEN
  payload = json_build_object('new', to_jsonb(NEW));
  ret = NEW;
END IF;

INSERT INTO log (id, created, category, message) VALUES
       (uuid_generate_v4(), now(), 'NOTIFY',
        json_build_object('channel', channel, 'payload', payload));

RETURN ret;

END;
$BODY$
LANGUAGE plpgsql VOLATILE;
""")

    op.execute("""
CREATE TRIGGER log_task_insert_update_or_delete
AFTER INSERT OR UPDATE OR DELETE ON task
FOR EACH ROW EXECUTE PROCEDURE log_insert_update_or_delete();
""")

    op.execute("""
CREATE TRIGGER log_assignment_insert_update_or_delete
AFTER INSERT OR UPDATE OR DELETE ON assignment
FOR EACH ROW EXECUTE PROCEDURE log_insert_update_or_delete();
""")


def downgrade():
    op.execute('DROP TRIGGER IF EXISTS log_task_insert_update_or_delete ON task;')
    op.execute('DROP TRIGGER IF EXISTS log_assignment_insert_update_or_delete ON assignment;')
    op.execute('DROP FUNCTION IF EXISTS log_insert_update_or_delete;')
