"""
Add none to detection occlusion enum

Revision ID: 8f88e10674f3
Revises: 7035ea2d9a55
Create Date: 2018-08-12 20:37:19.335053

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '8f88e10674f3'
down_revision = '7035ea2d9a55'
branch_labels = None
depends_on = None

# This is problematic because ALTER TYPE can't occur in a transaction.
# See https://stackoverflow.com/questions/14845203/altering-an-enum-field-using-alembic/16821396#16821396  # noqa
old_options = ('mostly', 'partially')
new_options = sorted(old_options + ('none',))

old_enum_type = sa.Enum(*old_options, name='detection_occlusion_enum')
new_enum_type = sa.Enum(*new_options, name='detection_occlusion_enum')
tmp_enum_type = sa.Enum(*new_options, name='_detection_occlusion_enum')


def upgrade():
    # Create a temporary enum type, convert and drop the old enum type.
    tmp_enum_type.create(op.get_bind(), checkfirst=False)
    op.execute('ALTER TABLE detection ALTER COLUMN occlusion TYPE _detection_occlusion_enum'
               ' USING occlusion::text::_detection_occlusion_enum')
    old_enum_type.drop(op.get_bind(), checkfirst=False)
    # Create and convert to the new enum type.
    new_enum_type.create(op.get_bind(), checkfirst=False)
    op.execute('ALTER TABLE detection ALTER COLUMN occlusion TYPE detection_occlusion_enum'
               ' USING occlusion::text::detection_occlusion_enum')
    tmp_enum_type.drop(op.get_bind(), checkfirst=False)


def downgrade():
    # Not worth the trouble.
    pass
