"""
rename artifact to answer

Revision ID: c163c1619520
Revises: a2edd5cb7071
Create Date: 2018-07-26 07:49:30.880815

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql as pg

# revision identifiers, used by Alembic.
revision = 'c163c1619520'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.rename_table('artifact', 'answer')
    op.drop_constraint('fk_artifact_assignment_id_assignment', 'answer')
    op.create_foreign_key('fk_answer_assignment_id_assignment', 'answer',
                          'assignment', ['assignment_id'], ['id'])
    op.add_column('answer', sa.Column('answer', pg.JSONB(), nullable=True))

    # This attempts to parse the "path" column as a json string
    # to support multiple rounds of upgrade/downgrade without
    # data loss.
    op.execute("""
    CREATE FUNCTION stumpf_answer_path_to_data(t text) returns json as $$
    BEGIN
        RETURN t::JSONB;
    EXCEPTION WHEN invalid_text_representation THEN
        RETURN json_build_object('path', t);
    END;
    $$ LANGUAGE plpgsql;
    UPDATE answer SET answer = stumpf_answer_path_to_data(path);
    DROP FUNCTION stumpf_answer_path_to_data;
    """)

    op.drop_column('answer', 'path')
    op.alter_column('answer', 'answer', nullable=False)


def downgrade():
    op.rename_table('answer', 'artifact')
    op.drop_constraint('fk_answer_assignment_id_assignment', 'artifact')
    op.create_foreign_key('fk_artifact_assignment_id_assignment', 'artifact',
                          'assignment', ['assignment_id'], ['id'])

    op.add_column('artifact', sa.Column('path', pg.TEXT(), nullable=True))
    op.execute("""UPDATE artifact SET path = answer::TEXT""")
    op.drop_column('artifact', 'answer')
    op.alter_column('artifact', 'path', nullable=False)
