"""
add arity answer functions

Revision ID: e2ce8102a19b
Revises: 55969209f3b9
Create Date: 2018-09-05 13:25:24.446464

"""
from alembic import op


# revision identifiers, used by Alembic.
revision = 'e2ce8102a19b'
down_revision = '55969209f3b9'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION arity_ground_truth(
            video_id_ uuid, start_ int, end_ int, activity_ activity_type_enum)
        RETURNS arity_adjudication_enum AS $$
            DECLARE
                start_frame int;
                end_frame int;
                fps int;
                activity_count bigint;
                ret arity_adjudication_enum;
            BEGIN
                SELECT frame_rate FROM video WHERE id = video_id_ LIMIT 1 INTO fps;
                start_frame = fps * start_;
                end_frame = fps * end_;
                SELECT COUNT(*) FROM activity WHERE video_id = video_id_
                    AND type_ = activity_
                    AND int4range(start_frame, end_frame) && frame_range
                INTO activity_count;

                IF(activity_count = 0) THEN
                    ret = '0 times';
                ELSIF(activity_count = 1) THEN
                    ret = '1 time';
                ELSE
                    ret = '2+ times';
                END IF;
                RETURN ret;
            END
        $$ LANGUAGE plpgsql STABLE;
    """)

    op.execute("""
        CREATE OR REPLACE FUNCTION arity_task_answer_is_correct(
            task_id_ uuid, activity_ activity_type_enum)
        RETURNS boolean AS $$
            DECLARE
                ground_truth arity_adjudication_enum;
                answer arity_adjudication_enum;
                ret boolean;
            BEGIN
                SELECT arity_ground_truth(video_id, "start", "end", activity_)
                FROM view_arity_task_answers WHERE id = task_id_ LIMIT 1 INTO ground_truth;
                EXECUTE format('SELECT %I FROM view_arity_task_answers WHERE id = %L LIMIT 1;',
                               activity_, task_id_)
                INTO answer;

                IF(answer = 'No agreement' OR answer is Null) THEN
                    ret = null;
                ELSIF(ground_truth = answer) THEN
                    ret = true;
                ELSE
                    ret = false;
                END IF;
                RETURN ret;
            END
        $$ LANGUAGE plpgsql STABLE;
    """)


def downgrade():
    op.execute('DROP FUNCTION IF EXISTS arity_ground_truth')
    op.execute('DROP FUNCTION IF EXISTS arity_task_answer_is_correct')
