"""
Add DIVA models

Revision ID: fb6c28f4a4c0
Revises: c163c1619520
Create Date: 2018-07-26 11:25:21.035195

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'fb6c28f4a4c0'
down_revision = 'c163c1619520'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('activity',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('video_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('type_', postgresql.ENUM('Sit_Down', 'Stand_Up', 'Carry_in_Hands', 'Carry_on_Back', 'Talk_on_Phone', 'Text_On_Phone', 'Read_Document', 'Vehicle_Moving', 'Vehicle_Stopping', 'Vehicle_Starting', 'Vehicle_Turning_Left', 'Vehicle_Turning_Right', 'Vehicle_UTurn', 'Object_Transfer', 'Pick_Up_Object', 'Set_Down_Object', 'Heavy_Carry', 'Hand_Interaction', 'Embrace_Interaction', 'Open_Facility_Door', 'Close_Facility_Door', 'Enter_Facility', 'Exit_Facility', 'Open_Vehicle_Door', 'Close_Vehicle_Door', 'Enter_Vehicle', 'Exit_Vehicle', 'Open_Trunk', 'Close_Trunk', 'Load_Vehicle', 'Unload_Vehicle', 'Vehicle_PicksUp_Person', 'Vehicle_DropsOff_Person', 'People_Talking', 'Joining_Queue', 'Riding', 'Purchasing', 'Laptop_Interaction', 'Abandon_Package', 'Casing_Facility', 'Theft', name='activity_type_enum'), nullable=False),  # noqa
                    sa.Column('frame_range', postgresql.INT4RANGE(), nullable=False),
                    sa.ForeignKeyConstraint(['video_id'], ['video.id'],
                                            name=op.f('fk_activity_video_id_video')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_activity')))
    op.create_table('track',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('video_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('type_', postgresql.ENUM('Person', 'Vehicle', 'Sky', 'Building', 'Shadows', 'Grass', 'Ground', 'Trees', 'Parking_Lot', 'Street', 'Sidewalk', 'Crosswalk', 'Intersection', 'Stairs', 'Background', 'Unannotated', 'Door', 'Tree', 'Receptacle', 'Dumpster', 'Parking_Meter', 'ATM', 'Umbrella', 'Construction_Barrier', 'Bike', 'Construction_Vehicle', 'Push_Pulled_Object', 'Prop', 'Animal', 'Articulated_Infrastructure', 'Other', name='track_type_enum'), nullable=False),  # noqa
                    sa.ForeignKeyConstraint(['video_id'], ['video.id'],
                                            name=op.f('fk_track_video_id_video')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_track')))
    op.create_table('activity_track',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('activity_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('track_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.ForeignKeyConstraint(['activity_id'], ['activity.id'],
                                            name=op.f('fk_activity_track_activity_id_activity')),
                    sa.ForeignKeyConstraint(['track_id'], ['track.id'],
                                            name=op.f('fk_activity_track_track_id_track')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_activity_track')))
    op.create_table('detection',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('video_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('track_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('source', postgresql.ENUM('truth', 'ground-truth',
                                                        'linear-interpolation',
                                                        name='detection_source_enum'),
                              nullable=False),
                    sa.Column('occlusion', postgresql.ENUM('mostly', 'partially',
                                                           name='detection_occlusion_enum'),
                              nullable=False),
                    sa.Column('frame', sa.Integer(), nullable=True),
                    sa.Column('bounds', postgresql.ARRAY(sa.Integer()), nullable=True),
                    sa.ForeignKeyConstraint(['track_id'], ['track.id'],
                                            name=op.f('fk_detection_track_id_track')),
                    sa.ForeignKeyConstraint(['video_id'], ['video.id'],
                                            name=op.f('fk_detection_video_id_video')),
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_detection')))

    def downgrade():
        op.drop_table('detection')
        op.drop_table('activity_track')
        op.drop_table('track')
        op.drop_table('activity')
