from datetime import datetime, timedelta
import os
import dotenv
import pytest

from stumpf.app import create_app
from stumpf.fakes import fake
from stumpf.models import db, Token


@pytest.fixture(autouse=True)
def mock_load_dotenv(monkeypatch):
    def do_nothing(*args, **kwargs):
        pass

    monkeypatch.setattr(dotenv, 'load_dotenv', do_nothing)


@pytest.fixture
def app():
    app = create_app({
        'ENV': 'testing',
        'SQLALCHEMY_DATABASE_URI': os.getenv(
            'SQLALCHEMY_DATABASE_URI', 'postgresql://postgres@localhost/stumpf_testdb')
    })

    with app.app_context():
        db.create_all()

    yield app

    with app.app_context():
        views = [
            'view_token', 'view_mturk_token', 'view_assignment_status_time',
            'view_assignment_detail', 'view_assignment_status_change_detail', 'view_sns_events']
        db.session.execute(f'DROP VIEW IF EXISTS {",".join(views)}')
        db.session.commit()
        db.drop_all()


@pytest.fixture
def auth(app):
    with app.app_context():
        user = fake.stumpf_user()
        token = Token(user=user, expires=datetime.utcnow() + timedelta(hours=1))
        db.session.add_all([user, token])
        db.session.commit()
        yield user, token


@pytest.fixture
def task(app, video, annotation_app):
    with app.app_context():
        user = fake.stumpf_user()
        db.session.add(user)
        db.session.commit()

        task = fake.stumpf_task(video.id, annotation_app.id, user.id)
        db.session.add(task)
        db.session.commit()

        yield task


@pytest.fixture
def video(app):
    with app.app_context():
        v = fake.stumpf_video()
        db.session.add(v)
        db.session.commit()
        yield v


@pytest.fixture
def annotation_app(app):
    with app.app_context():
        aa = fake.stumpf_annotation_app()
        db.session.add(aa)
        db.session.commit()
        yield aa
