import pytest

from stumpf.diva_fact_schemas import (
    ActivityFact, TrackDetectionFact, TrackFact
)
from stumpf.models import db, Fact


activity_fact = {
    'type': 'People_Talking'
}

track_fact = {
    'type': 'Person'
}

detection_fact = {
    'type': track_fact['type'],
    'source': 'ground-truth',
    'occlusion': 'none',
    'left': 10,
    'right': 20,
    'top': 10,
    'bottom': 20
}


# It would be better to yield the created facts from a fixture; however,
# there seems to be a bug in flask sqlalchemy where a model created in
# a particular session does not get loaded as the correct type when
# loaded in the *same* session.
#
# For example, this fails
@pytest.mark.skip('Fails for unknown reasons')
def test_query_activity(app, video):
    with app.app_context():
        activity, prov = video.get_fact_from_external_process(
            activity_fact, schema='grozper:activity')
        db.session.add_all([prov, activity])
        db.session.commit()
        assert isinstance(ActivityFact.query.first(), ActivityFact)


# but this succeeds
def test_query_activity_in_new_context(app, video):
    with app.app_context():
        activity, prov = video.get_fact_from_external_process(
            activity_fact, schema='grozper:activity')
        db.session.add_all([prov, activity])
        db.session.commit()
    with app.app_context():
        assert isinstance(ActivityFact.query.first(), ActivityFact)


@pytest.fixture
def diva_facts(app, video):
    with app.app_context():
        activity, prov = video.get_fact_from_external_process(
            activity_fact,
            schema='grozper:activity'
        )
        db.session.add_all([prov, activity])
        db.session.commit()
        track, prov = video.get_fact_from_external_process(
            {
                'activity_id': str(activity.id),
                **track_fact
            },
            schema='grozper:track'
        )
        db.session.add_all([prov, track])
        db.session.commit()
        detection, prov = video.get_fact_from_external_process(
            {
                'activity_id': track.fact['activity_id'],
                'track_id': str(track.id),
                **detection_fact
            },
            schema='grozper:track_detection'
        )
        db.session.add_all([prov, detection])
        db.session.commit()
    yield


def test_load_activity(app, diva_facts):
    with app.app_context():
        activity = ActivityFact.query.first()
        assert isinstance(activity, ActivityFact)
        assert len(activity.tracks) == 1


def test_load_track(app, diva_facts):
    with app.app_context():
        track = TrackFact.query.first()
        assert isinstance(track, TrackFact)
        assert len(track.detections) == 1


def test_load_detection(app, diva_facts):
    with app.app_context():
        detection = TrackDetectionFact.query.first()
        assert isinstance(detection, TrackDetectionFact)


def test_load_as_derived_type(app, diva_facts):
    with app.app_context():
        for fact in Fact.query.all():
            assert isinstance(fact, (ActivityFact, TrackFact, TrackDetectionFact))
