from stumpf.models import (
    db, Fact,
    PROV_BATCH_PROCESS_CREATED_FACT,
    PROV_EXTERNAL_PROCESS_CREATED_FACT,
    PROV_TASK_CREATED_FACT,
    Provenance
)


def test_video_get_fact_from_task(app, video, task):
    with app.app_context():
        fact, prov = video.get_fact_from_task(
            {'foo': 'bar'}, task,
            provenance={'some': 'metadata'})
        assert isinstance(fact, Fact)
        assert isinstance(prov, Provenance)
        db.session.add_all([prov, fact])
        db.session.commit()

        assert prov.relationship == PROV_TASK_CREATED_FACT
        assert prov.task_id is not None
        assert prov.task_id == task.id
        assert prov.fact_id == fact.id
        assert fact.fact == {'foo': 'bar'}
        assert prov.meta == {'some': 'metadata'}
        assert fact.frame_range.lower == 0
        assert fact.frame_range.upper == int(video.frame_rate * video.duration)
        assert fact.frame_range.upper_inc is False


def test_video_get_fact_from_external_process(app, video, task):
    with app.app_context():
        fact, prov = video.get_fact_from_external_process(
            {'foo': 'bar'}, frame_range=[10, 100],
            provenance={'some': 'metadata'})
        assert isinstance(fact, Fact)
        assert isinstance(prov, Provenance)
        db.session.add_all([prov, fact])
        db.session.commit()

        assert prov.relationship == PROV_EXTERNAL_PROCESS_CREATED_FACT
        assert prov.task_id is None
        assert prov.fact_id == fact.id
        assert fact.fact == {'foo': 'bar'}
        assert prov.meta == {'some': 'metadata'}
        assert fact.frame_range.lower == 10
        assert fact.frame_range.upper == 100
        assert fact.frame_range.upper_inc is False


def test_video_get_fact_from_batch_process(app, video, task):
    with app.app_context():
        fact, prov = video.get_fact_from_batch_process(
            {'foo': 'bar'},  provenance={'some': 'metadata'})
        assert isinstance(fact, Fact)
        assert isinstance(prov, Provenance)
        db.session.add_all([prov, fact])
        db.session.commit()

        assert prov.relationship == PROV_BATCH_PROCESS_CREATED_FACT
        assert prov.task_id is None
        assert prov.fact_id == fact.id
        assert fact.fact == {'foo': 'bar'}
        assert prov.meta == {'some': 'metadata'}
