from datetime import datetime, timedelta
import json

from mock import patch
import pytest

from stumpf.fakes import fake
from stumpf.models import Assignment, Task


def test_create_task_hit(app, auth, video, annotation_app):
    with patch('stumpf.views.task.create_hit') as create_hit:
        create_hit.return_value = {
            'HIT': {
                'HITId': 'some-hit-id',
                'HITGroupId': 'some-hit-group-id'
            }
        }

        with app.test_client() as client:
            resp = client.post(f'/api/v1/task/{video.id}/{annotation_app.id}/hit', json={
                'title': 'some-title',
                'description': 'some-description',
                'max_assignments': 10,
                'assignment_duration_in_seconds': 60,
                'lifetime_in_seconds': 60,
                'reward': 5
            }, headers={
                'X-Stumpf-Token': auth[1].token
            })

    assert resp.status_code == 201, resp.json

    task = Task.query.filter_by(hit_id='some-hit-id').first()

    assert task is not None

    assert len(task.status_changes) == 1
    assert task.status == 'Assignable'
