import axios from 'axios';
import MockAdapter from 'axios-mock-adapter';
import VueRouter from 'vue-router';
import Vuetify from 'vuetify';
import VueAxios from 'vue-axios';

import { mount, createLocalVue } from '@vue/test-utils';

import TasksView from '../../../src/components/TasksView';
import getToken from '../../../src/token';

const axiosInstance = axios.create({
  baseURL: '/api/v1',
  headers: {
    'X-Stumpf-token': getToken(),
    'Content-Type': 'application/json',
  },
});
const mock = new MockAdapter(axiosInstance);
mock.onGet(/.*/).reply(200, [], { 'resource-count': 0 });

const localVue = createLocalVue();
localVue.use(VueRouter);
localVue.use(Vuetify);
localVue.use(VueAxios, axiosInstance);

describe('TasksView.vue', () => {
  let app;

  // Fixes warning with certain vuetify components
  // https://forum.vuejs.org/t/vuetify-data-app-true-and-problems-rendering-v-dialog-in-unit-tests/27495/2
  beforeEach(() => {
    app = document.createElement('div');
    app.setAttribute('data-app', true);
    document.body.appendChild(app);
  });
  afterEach(() => {
    document.body.removeChild(app);
  });

  it('should render correct header', () => {
    const wrapper = mount(TasksView, {
      localVue,
    });

    expect(wrapper.find('.header-tasks-view').text()).equal('Tasks');
  });

  it('should render correct data table', () => {
    const wrapper = mount(TasksView, {
      localVue,
    });

    expect(wrapper.find('.header-tasks-view').text()).equal('Tasks');
  });

  it('should render correct create task button', () => {
    const wrapper = mount(TasksView, {
      localVue,
    });

    expect(wrapper.find('.tasks-btn-create-task').exists()).equal(true);
  });
});
