import axios from 'axios';
import MockAdapter from 'axios-mock-adapter';
import VueRouter from 'vue-router';
import Vuetify from 'vuetify';
import VueAxios from 'vue-axios';
import { mount, createLocalVue } from '@vue/test-utils';

import VideosView from '../../../src/components/VideosView';
import getToken from '../../../src/token';

const axiosInstance = axios.create({
  baseURL: '/api/v1',
  headers: {
    'X-Stumpf-token': getToken(),
    'Content-Type': 'application/json',
  },
});
const mock = new MockAdapter(axiosInstance);
mock.onGet(/.*/).reply(200, [], { 'resource-count': 0 });

const localVue = createLocalVue();
localVue.use(VueRouter);
localVue.use(Vuetify);
localVue.use(VueAxios, axiosInstance);

describe('VideosView.vue', () => {
  let app;

  // Fixes warning with certain vuetify components
  // https://forum.vuejs.org/t/vuetify-data-app-true-and-problems-rendering-v-dialog-in-unit-tests/27495/2
  beforeEach(() => {
    app = document.createElement('div');
    app.setAttribute('data-app', true);
    document.body.appendChild(app);
  });
  afterEach(() => {
    document.body.removeChild(app);
  });
  it('should render correct header', () => {
    const wrapper = mount(VideosView, {
      localVue,
    });
    expect(wrapper.find('.video-info-number').exists()).equal(true);
    expect(wrapper.find('.video-info-text').exists()).equal(true);
  });
});
